SystemOrganization addCategory: #'Seaside-Flow'!

!WAPlatform methodsFor: '*seaside-flow' stamp: 'jf 1/27/2009 12:17'!
suspendCallbackDo: aBlock
	"Evaluate aBlock, passing in a continuation that can be evaluated to answer to the place where this very method was called. This method is presumed to be executed only within the scope of an executing WAActionCallback and should, ideally, create a continuation that includes only the contexts that make up the callback execution (a partial continuation). Do not use this method, if you don't understand this comment."
	self subclassResponsibility! !

!WAPainterVisitor methodsFor: '*seaside-flow' stamp: 'jf 2/8/2009 10:52'!
visitTask: aTask
	self visitComponent: aTask! !

WAPainterVisitor subclass: #WATaskVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Flow'!

!WATaskVisitor methodsFor: 'actions' stamp: 'jf 2/8/2009 10:56'!
executeTask: aTask
	[ aTask answer: aTask go ] repeat! !

!WATaskVisitor methodsFor: 'visiting' stamp: 'jf 2/8/2009 15:01'!
visitTask: aTask
	super visitTask: aTask.
	(aTask activeComponent == aTask) ifTrue: [
		[ aTask execute ]
			on: WARenderNotification
			do: [ :n | ] ]! !

WARenderLoopMain subclass: #WAFlowMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Flow'!

!WAFlowMain class methodsFor: 'class initialization' stamp: 'jf 2/8/2009 15:53'!
initialize
	WAAdmin applicationDefaults
		at: #mainClass put: WAFlowMain;
		at: #actionPhaseContinuationClass put: WAFlowActionPhaseContinuation! !

!WAFlowMain methodsFor: 'processing' stamp: 'jf 2/8/2009 14:54'!
prepareRoot: aComponent
	super prepareRoot: aComponent.
	(WAVisiblePresenterGuide client: (WATaskVisitor new)) start: aComponent! !

!WARequestContext methodsFor: '*seaside-flow' stamp: 'jf 1/21/2009 10:51'!
redirect
	^ self respondAndContinue: [ :url | self redirectTo: url ]! !

!WARequestContext methodsFor: '*seaside-flow' stamp: 'lr 1/12/2009 09:07'!
redirectWithMessage: aString delay: aNumber 
	self respondAndContinue: [ :url | 
		self responseFactory
			refreshAfter: aNumber
			location: url
			message: aString ]! !

!WARequestContext methodsFor: '*seaside-flow' stamp: 'jf 1/27/2009 12:19'!
respondAndContinue: aBlock 
	^ WAPlatform current suspendCallbackDo: [ :cc |
		aBlock value: (self session actionUrlForContinuation: cc).
		self respond ]! !

WAActionPhaseContinuation subclass: #WAFlowActionPhaseContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Flow'!

!WAFlowActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 2/8/2009 12:25'!
runCallbacks
	super runCallbacks.
	(WAVisiblePresenterGuide client: (WATaskVisitor new)) start: root.! !

!WASession methodsFor: '*seaside-flow' stamp: 'jf 1/27/2009 12:24'!
inNewRenderLoopCall: aComponent
	^ WAPlatform current suspendCallbackDo: [ :cc | 
		aComponent onAnswer: cc.
		self inNewRenderLoopShow: aComponent ] ! !

!WAPackage class methodsFor: '*seaside-flow' stamp: 'jf 2/8/2009 15:55'!
seasideFlow
	^ self new
		name: 'Seaside-Flow';
		description: 'Define control flow as a continuous piece of code, with natural, linear control flow.';
		addDependency: 'Seaside-Core'; "Methods are added to WARequestContext"
		addDependency: 'Seaside-Component'; "Methods are added to WAComponent"
		addDependency: 'Seaside-RenderLoop'; "Methods are added to WASession that depend on RenderLoop"
		addDependency: 'Seaside-Environment'; "We currently us WAFlowMain class>>initialize to configure the application defaults to use out Main and ActionPhaseContinuation"
		yourself! !

!WAComponent methodsFor: '*seaside-flow' stamp: 'lr 1/24/2009 21:00'!
call: aComponent 
	"Pass control from the receiver to aComponent. The receiver will be temporarily replaced with aComponent. Code can return from here later on by sending #answer: to aComponent."

	^ self wait: [ :cc | self show: aComponent onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:03'!
chooseFrom: aCollection
	"Choose an item from the given aCollection. Answer the selected item."

	^ self wait: [ :cc | self chooseFrom: aCollection onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:04'!
chooseFrom: aCollection caption: aString
	"Choose an item from the given aCollection with caption aString. Answer the selected item."
	
	^ self wait: [ :cc | self chooseFrom: aCollection caption: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:04'!
confirm: aString
	"Display a yes/no dialog with caption aString. Answer true if the response is yes, false if no."

	^ self wait: [ :cc | self confirm: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:04'!
inform: aString
	"Display a dialog with aString to the user until he clicks the ok button."

	^ self wait: [ :cc | self inform: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString
	"Display an input dialog with the question aRequestString. Answer the string the user enters."

	^ self wait: [ :cc | self request: aRequestString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString default: aDefaultString
	"Display an input dialog with the question aRequestString and the default string aDefaultString. Answer the string the user enters."
	
	^ self wait: [ :cc | self request: aRequestString default: aDefaultString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString label: aLabelString
	"Display an input dialog with the question aRequestString and the button label aLabelString. Answer the string the user enters."
	
	^ self wait: [ :cc | self request: aRequestString label: aLabelString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString label: aLabelString default: aDefaultString
	"Display an input dialog with the question aRequestString, the button label aLabelString and the default string aDefaultString. Answer the string the user enters."

	^ self wait: [ :cc | self request: aRequestString label: aLabelString default: aDefaultString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow' stamp: 'jf 1/27/2009 12:18'!
wait: aBlock
	"Evaluate aBlock and pass in a continuation that can be evaluated to answer to the place where this very method was called. Note that after evaluating aBlock a re-rendering of the receiver is enfored. Do not use this method, if you don't understand this comment."

	^ WAPlatform current suspendCallbackDo: [ :cc |
		aBlock value: cc.
		WARenderNotification signal ]! !

WAComponent subclass: #WATask
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Flow'!

!WATask commentStamp: 'pmm 6/30/2007 19:02' prior: 0!
I am a subclass of WAComponent, specialized for defining workflow.  The difference between a task and a component is the following:

Both of them are reusable, embeddable, callable pieces of user interface. A component has state (instance variables), behavior (it may change its state, and it may also choose to display other components with #call:), and appearance (it renders HTML). A Task has only the first two - it doesn't render any HTML directly, but only through the components it calls. This is useful when what you want to encapsulate/embed/call is purely a process (show this component, then this one, then this one).

The key method for WATask is #go - as soon as a task is displayed, this method will get invoked, and will presumably #call: other components.

In terms of implementation, you can think of a WATask in the following way: it is a component which renders two things:
- a link whose callback invokes the #go method
- a header that immediately redirects to the URL of that link

Subclasses must not implement #renderContentOn:

An example can be found in WAConvenienceTest.
!

!WATask methodsFor: 'visiting' stamp: 'jf 2/8/2009 10:53'!
accept: aVisitor
	aVisitor visitTask: self! !

!WATask methodsFor: 'running' stamp: 'jf 2/8/2009 15:28'!
execute
	"Start the task running. Subclasses should usually implement #go to specify behaviour."
	
	[ self answer: self go ] repeat! !

!WATask methodsFor: 'running' stamp: 'jf 11/10/2003 16:13'!
go
	self subclassResponsibility! !
WAFlowMain initialize!
